/**
 * @brief   Device configuration structures and requets definitions
 * @file    ARM_Ethernet_Reader.h
 * @author  Ondrej Jerabek
 */

#ifndef ARM_ETHERNET_READER_H_
#define ARM_ETHERNET_READER_H_

#include <type.h>
#include <coremem.h>

// Service requests
#define RQ_SERVICE_SET_FREQ		0x01
#define RQ_SERVICE_SHUTDOWN		0x02
#define RQ_SERVICE_CTEST_START	0x03
#define RQ_SERVICE_CTEST_STOP	0x04
#define RQ_SERVICE_CTEST_GET_RESULT	0x05
#define RQ_SERVICE_GET_RSSI_TABLE 0x06

// --------------------------------------------------------------------------
/**
 * @brief Selection type of status structure
 */
enum e_st_type
{
	ST_DEVICE,			/**< Device status	*/
};

/**
 * @brief Selection type of configuration structure
 */
enum e_cfg_type
{
	CFG_DEVICE,			/**< Device configuration				*/
	CFG_MODULE1_SX,		/**< Semtech module 1 configuration		*/
	CFG_MODULE1_NTR,	/**< Nanotron module 1 configuration	*/
	CFG_MODULE1_DW,		/**< Decawave module 1 configuration	*/
	CFG_MODULE2_SX,		/**< Semtech module 2 configuration		*/
	CFG_MODULE2_NTR,	/**< Nanotron module 2 configuration	*/
	CFG_MODULE2_DW,		/**< Decawave module 2 configuration	*/
	CFG_ALL = 0xFF,		/**< All configuration selection	*/
};

// --------------------------------------------------------------------------
/**
 * @brief Device state structure version
 */
#define ST_VER_DEVICE		2

/**
 * @brief Device state structure
 */
typedef struct {
	STRUCT_HEADER header;			/**< Header of structure		*/
	uint32_t	uptime;				/**< Device up-time	[sec]		*/
	uint16_t	supply_voltage;		/**< Supply voltage (Before switching DC-DC converter) [mV]*/
	uint16_t	usb_voltage;		/**< Voltage on USB port [mV]	*/
	uint16_t	batt_voltage;		/**< Battery voltage [mV]		*/
	uint16_t	chrg_current;		/**< Charging current [mA]		*/
	TEMP		temperature;		/**< Temperature [0.01 °C]		*/

	unsigned
			charging: 1,			/**< Charging active flag		*/
			pwr_good: 1,			/**< External power flag		*/
			chrg_mode: 3,			/**< Type of charging mode		*/
			server_connected: 1,	/**< Server connection flag		*/
			reserved: 2;			/**< Reserved for future use	*/

	uint8_t		module1_type;		/**< Type of module in slot 1		*/
	uint8_t		module2_type;		/**< Type of module in slot 2		*/
	uint8_t		module1_rssi_back;	/**< RSSI background from module 1	*/
	uint8_t		module2_rssi_back;	/**< RSSI background from module 2	*/
} EthernetReader_State;
// --------------------------------------------------------------------------

/**
 * @brief Version of device configuration structure
 */
#define CFG_VER_DEVICE		1

/**
 * @brief Device configuration structure
 */
typedef struct {
	STRUCT_HEADER header;			/**< Header of structure				*/

	unsigned
		module1_on: 1,				/**< Enable power to module 1			*/
		module2_on: 1,				/**< Enable power to module 2			*/
		route_from_wireless1: 1,	/**< Routing all packet from module 1	*/
		route_from_wireless2: 1,	/**< Routing all packet from module 2	*/
		usb_enabled: 1,				/**< Enable USB communication.			*/
		nanotron_clk_en: 1,			/**< Enable 32 MHz clock for nanotron	*/
		reserved: 2;				/**< Reserved for future use			*/
} EthernetReader_Cfg;

// --------------------------------------------------------------------------

/**
 * @brief Version of Semtech configuration structure
 */
#define CFG_VER_SX		1

/**
 * @brief Semtech configuration structure
 */
typedef struct {
	STRUCT_HEADER header;			/**< Header of structure				*/

	unsigned long rf_frequency_1;	/**< Frequency of first configuration	*/
	unsigned long rf_frequency_2;	/**< Frequency of second configuration	*/
	unsigned char if_gain;			/**< Gain of receiver amplifier 		*/
	unsigned char rf_power;			/**< Transmitting power 				*/
	unsigned char vco_trim;			/**< VCO configuration 					*/ // TODO: should be loaded on startup from EEPROM on SX module
	unsigned
		listen_on_chn2: 1,			/**< Configuration selection 			*/
		clk_out: 1,					/**< Enable output clock (for measuring)*/
		reserved: 6;				/**< Reserved for future use			*/
} EthernetReader_SX_Cfg;
// --------------------------------------------------------------------------

/**
 * @brief Version of Nanotron configuration structure
 */
#define CFG_VER_NTR		1

/**
 * @brief Nanotron configuration structure
 */
typedef struct {
	STRUCT_HEADER header;		/**< Header of structure				*/

	unsigned char nano_pwr;		/**< Transmitting power 				*/
	unsigned char mode;			/**< Mode of Nanotron configuration		*/
} EthernetReader_NTR_Cfg;
// --------------------------------------------------------------------------

/**
 * @brief Version of Decawave configuration structure
 */
#define CFG_VER_DW		1

/**
 * @brief Decawave configuration structure
 * @note Not fully implemented yet
 */
typedef struct {
	STRUCT_HEADER header;		/**< Header of structure	*/
} EthernetReader_DW_Cfg;
// --------------------------------------------------------------------------

/**
 * @brief Version of configuration in SX module EEPROM
 */
#define CFG_VER_SX_MODULE		1

/**
 * @brief Configuration in SX module EEPROM
 */
typedef struct {
	STRUCT_HEADER header;		/**< Header of structure	*/

	unsigned char vco_trim;		/**< VCO configuration		*/
} Module_SX_Cfg;
// --------------------------------------------------------------------------

/**
 * @brief One slot configuration
 */
typedef struct {
	EthernetReader_SX_Cfg sx;
	EthernetReader_NTR_Cfg ntr;
	EthernetReader_DW_Cfg dw;
} ModuleCfg;

#endif /* ARM_ETHERNET_READER_H_ */
