/**
 * @brief   Function and structures defined in main program
 * @file    main.h
 * @author  Ondrej Jerabek
 */

#ifndef _MAIN_H
#define _MAIN_H

#include "config.h"
#include "chip.h"
#include "ARM_Ethernet_Reader.h"
#include "coremem.h"

// --------------------------------------------------------------------------
/**
 * @brief   Handling LED lighting
 * @param   var:	LED lighting timer
 * @param	led:	LED pin definition
 */
#define handle_led(var, led)	\
{								\
	if (var)					\
	{							\
		if (var != 0xffff)		\
			var--;				\
		led(1);					\
	}							\
	else						\
		led(0);					\
}

// --------------------------------------------------------------------------
/**
 * @brief   Handling timer decrement
 * @param   timer:	Handled timer
 */
#define handle_timer(timer)			\
{									\
	if (timer)						\
		timer--;					\
}

// --------------------------------------------------------------------------
/**
 * @brief   Handling timer decrement, loading and flag
 * @param   tmr:	Handled timer
 * @param	val:	Value loaded to timer when reaches zero
 * @param	flag:	Flag which is set when timer reaches zero
 */
#define handle_timer_flags(tmr, val, flag)	\
{										\
	if (tmr)							\
		tmr--;							\
	else {								\
		tmr = val;						\
		flag = true;					\
	}									\
}

// --------------------------------------------------------------------------
/**
 * @brief   Handling led blinking
 * @param   tmr:	LED lighting timer
 * @param	blink:	Perform LED blink
 * @param	invert:	Inverts LED blinking
 * @param	led:	LED pin definition
 */
#define handle_led_blink(tmr, blink, invert, led)		\
{														\
	if(tmr > 0)											\
	{													\
		tmr--;											\
		if (tmr > (TMR_LED / 2))						\
			led(!(invert));								\
		else											\
			led(invert);								\
	}													\
	else												\
	{													\
		led(invert);									\
		if (blink)										\
		{												\
			blink = false;								\
			tmr = TMR_LED;								\
		}												\
	}													\
}

// --------------------------------------------------------------------------
/**
 * @brief 	Flags
 */
typedef struct {
	volatile bool sec;					/**< Second flag. Set to 1 after each second						*/
	volatile bool ms100;				/**< 100 milliseconds flag. Set to 1 after each 100 milliseconds	*/
	volatile bool power_off;			/**< Power off flag. Set to 1 shutdown device						*/
	volatile bool update_cfg;			/**< Configuration update flag. Set to 1 updates all configuration	*/
	volatile bool wakeup_burst;			/**< Wakeup burst flag. When set, burst sending is active			*/
	volatile bool reinit_modules;		/**< Reinitialization modules flag. Set to 1 reinit. all modules	*/
	volatile bool usb_forced;			/**< USB forced flag. When is set, USB is connected and Ethernet disabled	*/
	volatile bool sx1_irq_detected;		/**< Semtech 1 interrupt flag. Set to 1 when interrupt detected		*/
	volatile bool sx2_irq_detected;		/**< Semtech 2 interrupt flag. Set to 1 when interrupt detected		*/
} Flag_struct;

/**
 * @brief 	Timers
 */
typedef struct {
	volatile unsigned int sec;			/**< Second timer. For handling second flag							*/
	volatile unsigned char ms100;		/**< 100 millisecond timer. For handling 100ms flag					*/
	volatile unsigned int timer32;		/**< 32 bit timer incremented each millisecond						*/
	volatile unsigned int buzz;			/**< Timer for beeping												*/
	volatile unsigned int button;		/**< Timer for measuring button press time							*/
	volatile unsigned int press_time;	/**< Result of button press time									*/
	volatile unsigned char bootloader;	/**< Timer for run bootloader										*/
	volatile unsigned int timesync_broadcast;	/**< Timer for broadcasting time through nanotron			*/
} Timer_struct;

/**
 * @brief 	Variables for LED lighting and blinking
 */
typedef struct {
	unsigned
			mod1_blink: 1,				/**< Module 1 LED blink. Set to 1 perform one LED blink		*/
			mod2_blink: 1,				/**< Module 2 LED blink. Set to 1 perform one LED blink		*/
			conn_blink: 1,				/**< Connection LED blink. Set to 1 perform one LED blink	*/
			reserved: 5;				/**< Reserved for future use								*/
	volatile unsigned int mod1;			/**< Timer for Module 1 LED lighting						*/
	volatile unsigned int mod2;			/**< Timer for Module 2 LED lighting						*/
	volatile unsigned int conn;			/**< Timer for Connection LED lighting						*/
	volatile unsigned int err;			/**< Timer for Error LED lighting							*/
	volatile unsigned int pwr;			/**< Timer for Power LED lighting							*/
	volatile unsigned int sys;			/**< Timer for System LED lighting							*/
} Leds_struct;
// --------------------------------------------------------------------------
extern unsigned char info[INFO_SIZE];

// --------------------------------------------------------------------------
extern TCoreRAM core;
extern Flag_struct flags;
extern Timer_struct tmr;
extern Leds_struct led;
extern EthernetReader_State st;
extern EthernetReader_Cfg cfg;
extern ModuleCfg module_cfg[2];

// --------------------------------------------------------------------------
/**
 * @brief   Reading 32bit system timer.
 * @return  Value of system timer in [ms]
 */
extern unsigned long read_system_timer(void);

/**
 * @brief   Initializing clock for USB peripheral
 * @return  Nothing
 * @note    Overrides clock initialization in USB library
 */
extern void Chip_USB_Init(void);

/**
 * @brief   Initializing 32 MHz clock output for nanotron module.
 * @param   enabled:	Enable clock output
 * @return  Nothing
 */
extern void SystemClkOut_init(bool enabled);

#endif // _MAIN_H
