/**
 * @file   modules.c
 * @author Ondrej Jerabek
 */


// Includes =====================================
#include "modules.h"
#include "rssi.h"
#include "type.h"
#include "nanotron.h"
#include <main.h>

#define SPI_INDEX	MODULES_SPI

#include <sx_rssi.h>
#include <spi.h>
#include <delay.h>
#include <pck.h>
#include <sx1211config.h>
#include <multisx1211.h>
#include <multisx1211_packet.h>
#include <ntrx_packet.h>

// Definitions ==================================
/**
 * @brief   Selecting semtech module on SPI bus
 * @param   module:	Module index
 */
#define sx_select(module)			\
{									\
	if (module == MODULE_1) 		\
		SX_CONFIG_SELECT1();		\
	else if (module == MODULE_2)	\
		SX_CONFIG_SELECT2();		\
}

/**
 * @brief   Unselecting semtech modules on SPI bus
 */
#define sx_unselect()				\
{									\
	SX_CONFIG_UNSELECT1();			\
	SX_CONFIG_UNSELECT2();			\
}

/**
 * @brief   Selecting decawave module on SPI bus
 * @param   module:	Module index
 */
#define dw_select(module)			\
{									\
	if (module == MODULE_1) 		\
		DW_CS_0(0);					\
	else if (module == MODULE_2)	\
		DW_CS_1(0);					\
}

/**
 * @brief   Unselecting decawave modules on SPI bus
 */
#define dw_unselect()				\
{									\
	DW_CS_0(1);						\
	DW_CS_1(1);						\
}

/**
 * @brief   Selecting nanotron module on SPI bus
 * @param   module:	Module index
 */
#define ntr_select(module)			\
{									\
	if (module == MODULE_1) 		\
		NTRX_1_CS(0);				\
	else if (module == MODULE_2)	\
		NTRX_2_CS(0);				\
}

/**
 * @brief   Unselecting nanotron modules on SPI bus
 */
#define ntr_unselect()				\
{									\
	NTRX_1_CS(1);					\
	NTRX_2_CS(1);					\
}

/**
 * @brief   Set reset pin on module to defined state
 * @param   module:	Module index
 * @param	state:	Reset pin state
 */
#define module_reset(module, state)	\
{									\
	if (module == MODULE_1) 		\
		MODULE1_RESET(state);		\
	else if (module == MODULE_2)	\
		MODULE2_RESET(state);		\
}

// Global variables =============================
MODULE_STRUCT modules;

// Functions ====================================
 /**
  * @brief   	Sending packet through selected module
  * @param		module:		Module index
  * @param[in]	pck:		Pointer to packet which will be send through selected module
  * @return  	Nothing
  */
void module_pck_send(enum e_module module, TPacket *pck)
{
	if (module == MODULE_1)
	{
		switch(modules.slot1_type)
		{
			case MODULE_SX:
				wireless_pck_send_1(pck);
				led.mod1_blink = true;
				break;
			case MODULE_DW:			// TODO: pck_send handler for this module
				break;
			case MODULE_NTR:
				ntrx_pck_send(nanotron_get_ptr(0), pck);
				led.mod1_blink = true;
				break;
			default:
				break;
		}
	}
	else if (module == MODULE_2)
	{
		switch(modules.slot2_type)
		{
			case MODULE_SX:
				wireless_pck_send_2(pck);
				led.mod2_blink = true;
				break;
			case MODULE_DW:			// TODO: pck_send handler for this module
				break;
			case MODULE_NTR:
				ntrx_pck_send(nanotron_get_ptr(1), pck);
				led.mod2_blink = true;
				break;
			default:
				break;
		}
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   	Sending and receiving packet through selected module
 * @param		module:		Module index
 * @param[in]	pckin:		Pointer to packet which will be send through selected module
 * @param[out]	pckout:		Pointer to packet buffer where will be put received packet
 * @param		timeout:	Timeout for answer waiting in milliseconds
 * @return  	Result of sending or receiveing packet
 */
enum e_send_receive_status module_pck_send_receive(enum e_module module, TPacket *pckin, TPacket *pckout, unsigned char timeout)
{
	enum e_send_receive_status status = SRS_TRANSMIT_ERROR;
	if (module == MODULE_1)
	{
		switch(modules.slot1_type)
		{
			case MODULE_SX:
				status = wireless_pck_send_receive_1(pckin, pckout, pckin->Pck_Type >> 6, timeout);
				led.mod1_blink = true;
				break;
			case MODULE_DW:			// TODO: pck_send handler for this module
				break;
			case MODULE_NTR:
				status = ntrx_pck_send_receive(nanotron_get_ptr(0), pckin, pckout, timeout);
				led.mod1_blink = true;
				break;
			default:
				break;
		}
	}
	else if (module == MODULE_2)
	{
		switch(modules.slot2_type)
		{
			case MODULE_SX:
				status = wireless_pck_send_receive_2(pckin, pckout, pckin->Pck_Type >> 6, timeout);
				led.mod2_blink = true;
				break;
			case MODULE_DW:			// TODO: pck_send handler for this module
				break;
			case MODULE_NTR:
				status = ntrx_pck_send_receive(nanotron_get_ptr(1), pckin, pckout, timeout);
				led.mod2_blink = true;
				break;
			default:
				break;
		}
	}
	return status;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Measuring Receive Signal Strength Intensity of received packet
 * @param		module:		Module index
 * @return  	Received signal strength
 * @note		Implemented only on Semtech module
 */
unsigned char module_get_rssi(enum e_module module)
{
	if (module == MODULE_1)
	{
		switch(modules.slot1_type)
		{
			case MODULE_SX:
				return sx_read_rssi(module);
				break;
			case MODULE_DW:
				return RSSI_NOT_SUPPORTED;
				break;
			case MODULE_NTR:
				return RSSI_NOT_SUPPORTED;
				break;
			default:
				return RSSI_NOT_SUPPORTED;
				break;
		}
	}
	else if (module == MODULE_2)
	{
		switch(modules.slot2_type)
		{
			case MODULE_SX:
				return sx_read_rssi(module);
				break;
			case MODULE_DW:
				return RSSI_NOT_SUPPORTED;
				break;
			case MODULE_NTR:
				return RSSI_NOT_SUPPORTED;
				break;
			default:
				return RSSI_NOT_SUPPORTED;
				break;
		}
	}

	return RSSI_NOT_SUPPORTED;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Set selected module to sleep mode
 * @param		module:		Module index
 * @return  	Nothing
 * @note		Implemented only on Semtech module and Nanotron module
 */
void module_set_sleep(enum e_module module)
{
	if (module == MODULE_1)
	{
		switch(modules.slot1_type)
		{
			case MODULE_SX:
				sx_change_mode_1(SXM_SLEEP);
				break;
			case MODULE_DW:		// TODO: Set DW to sleep
				break;
			case MODULE_NTR:
				nanotron_pwr_down(nanotron_get_ptr(0));
				break;
			default:
				break;
		}
		modules.slot1_type = MODULE_OFF;
	}
	else if (module == MODULE_2)
	{
		switch(modules.slot2_type)
		{
			case MODULE_SX:
				sx_change_mode_2(SXM_SLEEP);
				break;
			case MODULE_DW:		// TODO: Set DW to sleep
				break;
			case MODULE_NTR:
				nanotron_pwr_down(nanotron_get_ptr(1));
				break;
			default:
				break;
		}
		modules.slot2_type = MODULE_OFF;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   	Measuring Receive Signal Strength Intensity of background
 * @param		module:		Module index
 * @return  	Received signal strength
 * @note		Implemented only on Semtech module and Nanotron module
 */
unsigned char module_get_rssi_background(enum e_module module)
{
	if (module == MODULE_1)
	{
		switch(modules.slot1_type)
		{
			case MODULE_SX:
				return sx_read_rssi_background(module);
				break;
			case MODULE_DW:
				return RSSI_NOT_SUPPORTED;
				break;
			case MODULE_NTR:
				return NTRXGetRssi(nanotron_get_ptr(0));
				break;
			default:
				return RSSI_NOT_SUPPORTED;
				break;
		}
	}
	else if (module == MODULE_2)
	{
		switch(modules.slot2_type)
		{
			case MODULE_SX:
				return sx_read_rssi_background(module);
				break;
			case MODULE_DW:
				return RSSI_NOT_SUPPORTED;
				break;
			case MODULE_NTR:
				return NTRXGetRssi(nanotron_get_ptr(1));
				break;
			default:
				return RSSI_NOT_SUPPORTED;
				break;
		}
	}

	return RSSI_NOT_SUPPORTED;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Switching power to each module
 * @param		module:		Module index
 * @param		pwr_state:	Power state of module. 1 - Power on, 0 - Power off
 * @return  	Nothing
 */
void module_set_pwr(enum e_module module, unsigned char pwr_state)
{
	switch (module)
	{
		case MODULE_1:
			MODULE1_PWR(!pwr_state);
			if (!!pwr_state && modules.slot1_type == MODULE_OFF)
				modules.slot1_type = MODULE_NOT_INIT;
			else if (!pwr_state)
				modules.slot1_type = MODULE_OFF;
			break;

		case MODULE_2:
			MODULE2_PWR(!pwr_state);
			if (!!pwr_state && modules.slot2_type == MODULE_OFF)
				modules.slot2_type = MODULE_NOT_INIT;
			else if (!pwr_state)
				modules.slot2_type = MODULE_OFF;
			break;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   	Semtech module detection
 * @param		module:		Module index
 * @return  	@c true if module was detected
 */
bool module_sx_present(enum e_module module)
{
	unsigned char data;

	module_reset(module, 1);
	module_set_pwr(module, 0);		// Power off module
	delay(100);						// wait
	module_set_pwr(module, 1);		// and then power on
	delay(100);
	module_reset(module, 0);
	delay(1);

	sx_select(module);
	spi_read_write(0b01000000);		// Read SX reg 0
	data = spi_read();
	sx_unselect();
	if (data != 0b00101000)
		return false;

	sx_select(module);
	spi_read_write(0b01000010);		// Read SX reg 1
	data = spi_read();
	sx_unselect();
	if (data != 0b10001000)
		return false;

	sx_select(module);
	spi_read_write(0b01000100);		// Read SX reg 2
	data = spi_read();
	sx_unselect();
	if (data != 0b00000011)
		return false;

	sx_select(module);
	spi_read_write(0b01000110);		// Read SX reg 3
	data = spi_read();
	sx_unselect();
	if (data != 0b00000111)
		return false;

	return true;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Decawave module detection
 * @param		module:		Module index
 * @return  	@c true if module was detected
 */
bool module_dw_present(enum e_module module)
{
	unsigned char data[4];

	module_reset(module, 1);
	module_set_pwr(module, 0);		// Power off module
	delay(100);						// wait
	module_set_pwr(module, 1);		// and then power on
	delay(10);
	module_reset(module, 0);
	delay(1);

	dw_select(module);
	spi_read_write(0b00000000);		// Read DW reg 0
	data[0] = spi_read();
	data[1] = spi_read();
	data[2] = spi_read();
	data[3] = spi_read();
	dw_unselect();
	if (data[3] == 0xDE && data[2] == 0xCA && data[1] == 0x01 && data[0] == 0x30)
		return true;
	return false;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Nanotron module detection
 * @param		module:		Module index
 * @return  	@c true if module was detected
 */
bool module_ntr_present(enum e_module module)
{
	unsigned char data[2];

	module_reset(module, 1);
	delay(5);
	module_reset(module, 0);
	delay(1);

	ntr_select(module);
	spi_write(0x81);		// Write data
	spi_write(0x00);		// Addr 0
	spi_write(0xC3);		// MSB first, push-pull output
	ntr_unselect();

	delay_us(10);

	ntr_select(module);
	spi_write(0x02);		// Read 2 bytes
	spi_write(0x01);		// Read NTR reg 1, 2
	data[0] = spi_read_write(0xff);
	data[1] = spi_read_write(0xff);
	ntr_unselect();
	if (data[0] == 0x05 && data[1] == 0x01)
		return true;
	return false;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Detect type of module plugged into slot
 * @param		module:		Module index
 * @return  	Nothing
 */
void module_detect(enum e_module module)
{
	switch (module)
	{
		case MODULE_1:
			if (module_sx_present(module))
				modules.slot1_type = MODULE_SX;
			else if (module_dw_present(module))
				modules.slot1_type = MODULE_DW;
			else if (module_ntr_present(module))
				modules.slot1_type = MODULE_NTR;
			else
				modules.slot1_type = MODULE_NONE;
			break;

		case MODULE_2:
			if (module_sx_present(module))
				modules.slot2_type = MODULE_SX;
			else if (module_dw_present(module))
				modules.slot2_type = MODULE_DW;
			else if (module_ntr_present(module))
				modules.slot2_type = MODULE_NTR;
			else
				modules.slot2_type = MODULE_NONE;
			break;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   	Set correct pin configuration for specified module type
 * @param		module:		Module index
 * @param		type:		Type of module plugged into slot
 * @return  	Nothing
 */
void module_configure_pins(enum e_module module, enum e_module_type type)
{
	if (module == MODULE_1)
	{
		switch (type)
		{
			case MODULE_SX:
				IRQ_01_DIR(0);
				IRQ_11_DIR(0);
				SX_EE_CS1_DIR(1);
				SX_EE_CS1(1);
				SX_PLLLOCK1_DIR(0);
				Chip_GPIOINT_SetIntRising(LPC_GPIOINT, GPIOINT_PORT2,
						Chip_GPIOINT_GetIntRising(LPC_GPIOINT, GPIOINT_PORT2) | (1 << IRQ01_PIN) | (1 << IRQ11_PIN));	// povoleni preruseni od SX1211 - ISM modul IRQ0, IRQ1
				Chip_GPIOINT_SetIntFalling(LPC_GPIOINT, GPIOINT_PORT2, 0);	// povoleni preruseni od SX1211 - ISM modul IRQ0, IRQ1
				NVIC_EnableIRQ(GPIO_IRQn);									// povoleni preruseni od celeho bloku
				break;
			case MODULE_DW:
				// TODO: Specific pin configuration for DW
				break;

			case MODULE_NTR:
				NTRX_1_PAEN_DIR(1);
				NTRX_1_HGM_DIR(1);
				NTRX_1_IRQ_DIR(0);
				NTRX_1_VCC_DIR(0);
				NTRX_1_D3_DIR(0);
				break;
			default:
				break;
		}
	}
	else if (module == MODULE_2)
	{
		switch (type)
		{
			case MODULE_SX:
				IRQ_02_DIR(0);
				IRQ_12_DIR(0);
				SX_EE_CS2_DIR(1);
				SX_EE_CS2(1);
				SX_PLLLOCK2_DIR(0);
				Chip_GPIOINT_SetIntRising(LPC_GPIOINT, GPIOINT_PORT2,
						Chip_GPIOINT_GetIntRising(LPC_GPIOINT, GPIOINT_PORT2) | (1 << IRQ02_PIN) | (1 << IRQ12_PIN));	// povoleni preruseni od SX1211 - ISM modul IRQ0, IRQ1
				Chip_GPIOINT_SetIntFalling(LPC_GPIOINT, GPIOINT_PORT2, 0);	// povoleni preruseni od SX1211 - ISM modul IRQ0, IRQ1
				NVIC_EnableIRQ(GPIO_IRQn);									// povoleni preruseni od celeho bloku
				break;
			case MODULE_DW:
				// TODO: Specific pin configuration for DW
				break;
			case MODULE_NTR:
				NTRX_2_PAEN_DIR(1);
				NTRX_2_HGM_DIR(1);
				NTRX_2_IRQ_DIR(0);
				NTRX_2_VCC_DIR(0);
				NTRX_2_D3_DIR(0);
				break;
			default:
				break;
		}
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   	Initialize all modules in slots
 * @return  	Nothing
 */
void modules_init(void)
{
	SystemClkOut_init(1);	// Enable nanotron clk out for initialization

	modules.slot1_type = MODULE_OFF;
	modules.slot2_type = MODULE_OFF;
	module_set_pwr(MODULE_1, 1);
	module_set_pwr(MODULE_2, 1);
	// Initialize module 1
	if (modules.slot1_type == MODULE_NOT_INIT)
	{
		module_detect(MODULE_1);
		switch (modules.slot1_type)
		{
			case MODULE_SX:
				module_configure_pins(MODULE_1, MODULE_SX);
				sx_init_1();
				sx_change_mode_1(SXM_RX);
				break;

			case MODULE_DW:
				module_configure_pins(MODULE_1, MODULE_DW);
				// TODO: Decawave init
				break;

			case MODULE_NTR:
				module_configure_pins(MODULE_1, MODULE_NTR);
				nanotron_init(0);
				nanotron_rx_enable(nanotron_get_ptr(0), 1);
				nanotron_enable_promiscuous(0, 1);
				break;

			default:
				MODULE1_PWR(1);	// Set empty slot off
				break;
		}

		if (!cfg.module1_on)	// Slot disabled -> set current module to sleep, but keep powered
			module_set_sleep(MODULE_1);
	}

	// Initialize module 2
	if (modules.slot2_type == MODULE_NOT_INIT)
	{
		module_detect(MODULE_2);
		switch (modules.slot2_type)
		{
			case MODULE_SX:
				module_configure_pins(MODULE_2, MODULE_SX);
				sx_init_2();
				sx_change_mode_2(SXM_RX);
				break;

			case MODULE_DW:
				module_configure_pins(MODULE_2, MODULE_DW);
				// TODO: Decawave init
				break;

			case MODULE_NTR:
				module_configure_pins(MODULE_2, MODULE_NTR);
				nanotron_init(1);
				nanotron_rx_enable(nanotron_get_ptr(1), 1);
				nanotron_enable_promiscuous(1, 1);
				break;

			default:
				MODULE2_PWR(1);	// Set empty slot off
				break;
		}

		if (!cfg.module2_on)	// Slot disabled -> set current module to sleep
			module_set_sleep(MODULE_2);
	}

	flags.update_cfg = true;
	st.module1_type = modules.slot1_type;
	st.module2_type = modules.slot2_type;
}
// --------------------------------------------------------------------------
